@@ -1,20 +1,20 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from __future__ import annotations
 
-from typing import ClassVar, Mapping
+from typing import AsyncGenerator, AsyncIterable, ClassVar, Literal, Mapping, Union
 from enum import Enum
 from json.decoder import JSONDecodeError
 from time import time
 from urllib.parse import quote as urllib_quote, urljoin as urllib_join
 import asyncio
+import inspect
 import json
 import logging
 import platform
-import sys
 
 from aiohttp import ClientSession, __version__ as aiohttp_version
 from aiohttp.client_exceptions import ClientError, ContentTypeError
@@ -25,11 +25,6 @@ from mautrix.errors import MatrixConnectionError, MatrixRequestError, make_reque
 from mautrix.util.logging import TraceLogger
 from mautrix.util.opt_prometheus import Counter
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
 if __optional_imports__:
     # Safe to import, but it's not actually needed, so don't force-import the whole types module.
     from mautrix.types import JSON
@@ -52,9 +47,8 @@ class APIPath(Enum):
     These don't start with a slash so they can be used nicely with yarl.
     """
 
-    CLIENT = "_matrix/client/r0"
-    CLIENT_UNSTABLE = "_matrix/client/unstable"
-    MEDIA = "_matrix/media/r0"
+    CLIENT = "_matrix/client"
+    MEDIA = "_matrix/media"
     SYNAPSE_ADMIN = "_synapse/admin"
 
     def __repr__(self):
@@ -88,8 +82,8 @@ class PathBuilder:
         >>> from mautrix.api import Path
         >>> room_id = "!foo:example.com"
         >>> event_id = "$bar:example.com"
-        >>> str(Path.rooms[room_id].event[event_id])
-        "_matrix/client/r0/rooms/%21foo%3Aexample.com/event/%24bar%3Aexample.com"
+        >>> str(Path.v3.rooms[room_id].event[event_id])
+        "_matrix/client/v3/rooms/%21foo%3Aexample.com/event/%24bar%3Aexample.com"
     """
 
     def __init__(self, path: str | APIPath = "") -> None:
@@ -129,40 +123,39 @@ class PathBuilder:
             return self
         return PathBuilder(f"{self.path}/{self._quote(str(append))}")
 
+    def replace(self, find: str, replace: str) -> PathBuilder:
+        return PathBuilder(self.path.replace(find, replace))
+
 
 ClientPath = PathBuilder(APIPath.CLIENT)
 ClientPath.__doc__ = """
-A path builder with the standard client r0 prefix ( ``/_matrix/client/r0``, :attr:`APIPath.CLIENT`)
+A path builder with the standard client prefix ( ``/_matrix/client``, :attr:`APIPath.CLIENT`).
 """
 Path = PathBuilder(APIPath.CLIENT)
 Path.__doc__ = """A shorter alias for :attr:`ClientPath`"""
-UnstableClientPath = PathBuilder(APIPath.CLIENT_UNSTABLE)
-UnstableClientPath.__doc__ = """
-A path builder for client endpoints that haven't reached the spec yet
-(``/_matrix/client/unstable``, :attr:`APIPath.CLIENT_UNSTABLE`)
-"""
 MediaPath = PathBuilder(APIPath.MEDIA)
 MediaPath.__doc__ = """
-A path builder for standard media r0 paths (``/_matrix/media/r0``, :attr:`APIPath.MEDIA`)
+A path builder with the standard media prefix (``/_matrix/media``, :attr:`APIPath.MEDIA`)
 
 Examples:
     >>> from mautrix.api import MediaPath
-    >>> str(MediaPath.config)
-    "_matrix/media/r0/config"
+    >>> str(MediaPath.v3.config)
+    "_matrix/media/v3/config"
 """
 SynapseAdminPath = PathBuilder(APIPath.SYNAPSE_ADMIN)
 SynapseAdminPath.__doc__ = """
 A path builder for synapse-specific admin API paths
-(``/_synapse/admin/v1``, :attr:`APIPath.SYNAPSE_ADMIN`)
+(``/_synapse/admin``, :attr:`APIPath.SYNAPSE_ADMIN`)
 
 Examples:
     >>> from mautrix.api import SynapseAdminPath
     >>> user_id = "@user:example.com"
-    >>> str(SynapseAdminPath.users[user_id]/login)
+    >>> str(SynapseAdminPath.v1.users[user_id]/login)
     "_synapse/admin/v1/users/%40user%3Aexample.com/login"
 """
 
 _req_id = 0
+AsyncBody = AsyncGenerator[Union[bytes, bytearray, memoryview], None]
 
 
 def _next_global_req_id() -> int:
@@ -171,6 +164,12 @@ def _next_global_req_id() -> int:
     return _req_id
 
 
+async def _async_iter_bytes(data: bytearray | bytes, chunk_size: int = 1024**2) -> AsyncBody:
+    with memoryview(data) as mv:
+        for i in range(0, len(data), chunk_size):
+            yield mv[i : i + chunk_size]
+
+
 class HTTPAPI:
     """HTTPAPI is a simple asyncio Matrix API request sender."""
 
@@ -237,7 +236,7 @@ class HTTPAPI:
         self,
         method: Method,
         url: URL,
-        content: bytes | str,
+        content: bytes | bytearray | str | AsyncBody,
         query_params: dict[str, str],
         headers: dict[str, str],
     ) -> JSON:
@@ -265,16 +264,23 @@ class HTTPAPI:
         self,
         method: Method,
         path: PathBuilder,
-        content: str | bytes,
+        content: str | bytes | bytearray | AsyncBody,
         orig_content,
         query_params: dict[str, str],
+        headers: dict[str, str],
         req_id: int,
     ) -> None:
         if not self.log:
             return
-        log_content = content if not isinstance(content, bytes) else f"<{len(content)} bytes>"
+        if isinstance(content, (bytes, bytearray)):
+            log_content = f"<{len(content)} bytes>"
+        elif inspect.isasyncgen(content):
+            size = headers.get("Content-Length", None)
+            log_content = f"<{size} async bytes>" if size else f"<stream with unknown length>"
+        else:
+            log_content = content
         as_user = query_params.get("user_id", None)
-        level = 1 if path == Path.sync else 5
+        level = 1 if path == Path.v3.sync else 5
         self.log.log(
             level,
             f"{method}#{req_id} /{path} {log_content}".strip(" "),
@@ -304,11 +310,12 @@ class HTTPAPI:
         self,
         method: Method,
         path: PathBuilder | str,
-        content: dict | list | bytes | str | None = None,
+        content: dict | list | bytes | bytearray | str | AsyncBody | None = None,
         headers: dict[str, str] | None = None,
         query_params: Mapping[str, str] | None = None,
         retry_count: int | None = None,
         metrics_method: str = "",
+        min_iter_size: int = 25 * 1024 * 1024,
     ) -> JSON:
         """
         Make a raw Matrix API request.
@@ -325,6 +332,9 @@ class HTTPAPI:
             retry_count: Number of times to retry if the homeserver isn't reachable.
                          Defaults to :attr:`default_retry_count`.
             metrics_method: Name of the method to include in Prometheus timing metrics.
+            min_iter_size: If the request body is larger than this value, it will be passed to
+                           aiohttp as an async iterable to stop it from copying the whole thing
+                           in memory.
 
         Returns:
             The parsed response JSON.
@@ -351,12 +361,19 @@ class HTTPAPI:
 
         if retry_count is None:
             retry_count = self.default_retry_count
+        if inspect.isasyncgen(content):
+            # Can't retry with non-static body
+            retry_count = 0
+        do_fake_iter = content and hasattr(content, "__len__") and len(content) > min_iter_size
+        if do_fake_iter:
+            headers["Content-Length"] = str(len(content))
         backoff = 4
         while True:
-            self._log_request(method, path, content, orig_content, query_params, req_id)
+            self._log_request(method, path, content, orig_content, query_params, headers, req_id)
             API_CALLS.labels(method=metrics_method).inc()
+            req_content = _async_iter_bytes(content) if do_fake_iter else content
             try:
-                return await self._send(method, full_url, content, query_params, headers or {})
+                return await self._send(method, full_url, req_content, query_params, headers or {})
             except MatrixRequestError as e:
                 API_CALLS_FAILED.labels(method=metrics_method).inc()
                 if retry_count > 0 and e.http_status in (502, 503, 504):
@@ -387,7 +404,10 @@ class HTTPAPI:
         return f"mautrix-python_R{self.txn_id}@T{int(time() * 1000)}"
 
     def get_download_url(
-        self, mxc_uri: str, download_type: Literal["download", "thumbnail"] = "download"
+        self,
+        mxc_uri: str,
+        download_type: Literal["download", "thumbnail"] = "download",
+        file_name: str | None = None,
     ) -> URL:
         """
         Get the full HTTP URL to download a ``mxc://`` URI.
@@ -395,6 +415,7 @@ class HTTPAPI:
         Args:
             mxc_uri: The MXC URI whose full URL to get.
             download_type: The type of download ("download" or "thumbnail").
+            file_name: Optionally, a file name to include in the download URL.
 
         Returns:
             The full HTTP URL.
@@ -403,11 +424,34 @@ class HTTPAPI:
             ValueError: If `mxc_uri` doesn't begin with ``mxc://``.
 
         Examples:
-            >>> api = HTTPAPI(...)
+            >>> api = HTTPAPI(base_url="https://matrix-client.matrix.org", ...)
             >>> api.get_download_url("mxc://matrix.org/pqjkOuKZ1ZKRULWXgz2IVZV6")
-            "https://matrix.org/_matrix/media/r0/download/matrix.org/pqjkOuKZ1ZKRULWXgz2IVZV6"
+            "https://matrix-client.matrix.org/_matrix/media/v3/download/matrix.org/pqjkOuKZ1ZKRULWXgz2IVZV6"
+            >>> api.get_download_url("mxc://matrix.org/pqjkOuKZ1ZKRULWXgz2IVZV6", file_name="hello.png")
+            "https://matrix-client.matrix.org/_matrix/media/v3/download/matrix.org/pqjkOuKZ1ZKRULWXgz2IVZV6/hello.png"
+        """
+        server_name, media_id = self.parse_mxc_uri(mxc_uri)
+        url = self.base_url / str(APIPath.MEDIA) / "v3" / download_type / server_name / media_id
+        if file_name:
+            url /= file_name
+        return url
+
+    @staticmethod
+    def parse_mxc_uri(mxc_uri: str) -> tuple[str, str]:
+        """
+        Parse a ``mxc://`` URI.
+
+        Args:
+            mxc_uri: The MXC URI to parse.
+
+        Returns:
+            A tuple containing the server and media ID of the MXC URI.
+
+        Raises:
+            ValueError: If `mxc_uri` doesn't begin with ``mxc://``.
         """
         if mxc_uri.startswith("mxc://"):
-            return self.base_url / str(APIPath.MEDIA) / download_type / mxc_uri[6:]
+            server_name, media_id = mxc_uri[6:].split("/")
+            return server_name, media_id
         else:
             raise ValueError("MXC URI did not begin with `mxc://`")
