@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,8 +8,7 @@ from __future__ import annotations
 from typing import Any, Awaitable, Iterable
 from urllib.parse import quote as urllib_quote
 
-from mautrix import __optional_imports__
-from mautrix.api import Method, Path, UnstableClientPath
+from mautrix.api import Method, Path
 from mautrix.client import ClientAPI, StoreUpdatingAPI
 from mautrix.errors import (
     IntentError,
@@ -47,13 +46,6 @@ from mautrix.util.logging import TraceLogger
 
 from .. import api as as_api, state_store as ss
 
-try:
-    import magic
-except ImportError:
-    if __optional_imports__:
-        raise
-    magic = None
-
 
 def quote(*args, **kwargs):
     return urllib_quote(*args, **kwargs, safe="")
@@ -77,6 +69,7 @@ ENSURE_REGISTERED_METHODS = (
     ClientAPI.search_users,
     ClientAPI.set_displayname,
     ClientAPI.set_avatar_url,
+    ClientAPI.unstable_create_mxc,
     ClientAPI.upload_media,
     ClientAPI.send_receipt,
     ClientAPI.set_fully_read_marker,
@@ -199,6 +192,7 @@ class IntentAPI(StoreUpdatingAPI):
         self,
         room_id: RoomID,
         user_id: UserID,
+        reason: str | None = None,
         check_cache: bool = False,
         extra_content: dict[str, Any] | None = None,
     ) -> None:
@@ -218,6 +212,8 @@ class IntentAPI(StoreUpdatingAPI):
         Args:
             room_id: The ID of the room to which to invite the user.
             user_id: The fully qualified user ID of the invitee.
+            reason: The reason the user was invited. This will be supplied as the ``reason`` on
+                the `m.room.member`_ event.
             check_cache: If ``True``, the function will first check the state store, and not do
                          anything if the state store says the user is already invited or joined.
             extra_content: Additional properties for the invite event content.
@@ -230,7 +226,9 @@ class IntentAPI(StoreUpdatingAPI):
                 await self.state_store.get_membership(room_id, user_id) not in ok_states
             )
             if do_invite:
-                await super().invite_user(room_id, user_id, extra_content=extra_content)
+                await super().invite_user(
+                    room_id, user_id, reason=reason, extra_content=extra_content
+                )
                 await self.state_store.invited(room_id, user_id)
         except MatrixRequestError as e:
             if e.errcode == "M_FORBIDDEN" and "is already in the room" in e.message:
@@ -460,7 +458,7 @@ class IntentAPI(StoreUpdatingAPI):
         Returns:
             All the event IDs generated, plus a batch ID that can be passed back to this method.
         """
-        path = UnstableClientPath["org.matrix.msc2716"].rooms[room_id].batch_send
+        path = Path.unstable["org.matrix.msc2716"].rooms[room_id].batch_send
         query = {"prev_event_id": prev_event_id}
         if batch_id:
             query["batch_id"] = batch_id
@@ -540,7 +538,7 @@ class IntentAPI(StoreUpdatingAPI):
             "inhibit_login": True,
         }
         query_params = {"kind": "user"}
-        return self.api.request(Method.POST, Path.register, content, query_params=query_params)
+        return self.api.request(Method.POST, Path.v3.register, content, query_params=query_params)
 
     async def ensure_registered(self) -> None:
         """
