@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -118,7 +118,7 @@ class AppServiceServerMixin:
 
     @staticmethod
     def _get_with_fallback(
-        json: Dict[str, Any], field: str, unstable_prefix: str, default: Any = None
+        json: dict[str, Any], field: str, unstable_prefix: str, default: Any = None
     ) -> Any:
         try:
             return json.pop(field)
