@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -73,6 +73,7 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
         copy("homeserver.connection_limit")
         copy("homeserver.status_endpoint")
         copy("homeserver.message_send_checkpoint_endpoint")
+        copy("homeserver.async_media")
 
         copy("appservice.address")
         copy("appservice.hostname")
@@ -124,18 +125,12 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
             if "bridge.alias_template" in self
             else None
         )
-        group_id = (
-            {"group_id": self["appservice.community_id"]}
-            if self["appservice.community_id"]
-            else {}
-        )
 
         return {
             "users": [
                 {
                     "exclusive": True,
                     "regex": re.escape(f"@{username_format}:{homeserver}").replace(regex_ph, ".*"),
-                    **group_id,
                 }
             ],
             "aliases": [
