@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -170,7 +170,7 @@ class CustomPuppetMixin(ABC):
             else:
                 raise AutologinError(f"No homeserver URL configured for {server}")
         password = hmac.new(secret, mxid.encode("utf-8"), hashlib.sha512).hexdigest()
-        url = base_url / str(Path.login)
+        url = base_url / str(Path.v3.login)
         resp = await cls.az.http_session.post(
             url,
             data=json.dumps(
