@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,18 +12,11 @@ import sys
 from mautrix import __optional_imports__
 from mautrix.appservice import AppService
 from mautrix.client import Client, SyncStore
-from mautrix.crypto import (
-    CryptoStore,
-    DeviceIdentity,
-    OlmMachine,
-    PgCryptoStore,
-    RejectKeyShare,
-    StateStore,
-    TrustState,
-)
+from mautrix.crypto import CryptoStore, OlmMachine, PgCryptoStore, RejectKeyShare, StateStore
 from mautrix.errors import EncryptionError, SessionNotFound
 from mautrix.types import (
     JSON,
+    DeviceIdentity,
     EncryptedEvent,
     EncryptedMegolmEventContent,
     EventFilter,
@@ -39,7 +32,9 @@ from mautrix.types import (
     Serializable,
     StateEvent,
     StateFilter,
+    TrustState,
 )
+from mautrix.util.async_db import Database, DatabaseException
 from mautrix.util.logging import TraceLogger
 
 from .. import bridge as br
@@ -52,13 +47,6 @@ except ImportError:
         raise
     UserProfile = None
 
-try:
-    from mautrix.util.async_db import Database
-except ImportError:
-    if __optional_imports__:
-        raise
-    Database = None
-
 
 class EncryptionManager:
     loop: asyncio.AbstractEventLoop
@@ -227,16 +215,17 @@ class EncryptionManager:
 
     async def start(self) -> None:
         flows = await self.client.get_login_flows()
-        flow = flows.get_first_of_type(LoginType.APPSERVICE, LoginType.UNSTABLE_APPSERVICE)
-        if flow is None:
+        if not flows.supports_type(LoginType.APPSERVICE):
             self.log.critical(
-                "Encryption enabled in config, but homeserver does not "
-                "advertise appservice login"
+                "Encryption enabled in config, but homeserver does not support appservice login"
             )
             sys.exit(30)
-        self.log.debug(f"Logging in with bridge bot user (using login type {flow.type.value})")
+        self.log.debug("Logging in with bridge bot user")
         if self.crypto_db:
-            await self.crypto_db.start()
+            try:
+                await self.crypto_db.start()
+            except Exception as e:
+                self.bridge._log_db_error(e)
         await self.crypto_store.open()
         device_id = await self.crypto_store.get_device_id()
         if device_id:
@@ -245,7 +234,7 @@ class EncryptionManager:
         # It'll get overridden after the login
         self.client.api.token = self.az.as_token
         await self.client.login(
-            login_type=flow.type,
+            login_type=LoginType.APPSERVICE,
             device_name=self.device_name,
             device_id=device_id,
             store_access_token=True,
