@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,6 +6,7 @@
 from __future__ import annotations
 
 from typing import Awaitable
+import json
 
 from mautrix.api import Method, Path
 from mautrix.errors import MatrixResponseError
@@ -32,6 +33,7 @@ from mautrix.types import (
     ReactionEventContent,
     RelatesTo,
     RelationType,
+    RoomEventFilter,
     RoomID,
     Serializable,
     SerializerError,
@@ -41,7 +43,6 @@ from mautrix.types import (
     TextMessageEventContent,
     UserID,
 )
-from mautrix.types.event.state import state_event_content_map
 from mautrix.util.formatter import parse_html
 
 from .base import BaseClientAPI
@@ -95,7 +96,7 @@ class EventMethods(BaseClientAPI):
         if set_presence:
             request["set_presence"] = str(set_presence)
         return self.api.request(
-            Method.GET, Path.sync, query_params=request, retry_count=0, metrics_method="sync"
+            Method.GET, Path.v3.sync, query_params=request, retry_count=0, metrics_method="sync"
         )
 
     # endregion
@@ -117,7 +118,7 @@ class EventMethods(BaseClientAPI):
             The event.
         """
         content = await self.api.request(
-            Method.GET, Path.rooms[room_id].event[event_id], metrics_method="getEvent"
+            Method.GET, Path.v3.rooms[room_id].event[event_id], metrics_method="getEvent"
         )
         try:
             return Event.deserialize(content)
@@ -147,13 +148,12 @@ class EventMethods(BaseClientAPI):
         """
         content = await self.api.request(
             Method.GET,
-            Path.rooms[room_id].state[event_type][state_key],
+            Path.v3.rooms[room_id].state[event_type][state_key],
             metrics_method="getStateEvent",
         )
+        content["__mautrix_event_type"] = event_type
         try:
-            return state_event_content_map[event_type].deserialize(content)
-        except KeyError:
-            return Obj(**content)
+            return StateEvent.deserialize_content(content)
         except SerializerError as e:
             raise MatrixResponseError("Invalid state event in response") from e
 
@@ -170,7 +170,7 @@ class EventMethods(BaseClientAPI):
             A list of state events with the most recent of each event_type/state_key pair.
         """
         content = await self.api.request(
-            Method.GET, Path.rooms[room_id].state, metrics_method="getState"
+            Method.GET, Path.v3.rooms[room_id].state, metrics_method="getState"
         )
         try:
             return [StateEvent.deserialize(event) for event in content]
@@ -213,7 +213,7 @@ class EventMethods(BaseClientAPI):
             query["not_membership"] = not_membership.value
         content = await self.api.request(
             Method.GET,
-            Path.rooms[room_id].members,
+            Path.v3.rooms[room_id].members,
             query_params=query,
             metrics_method="getMembers",
         )
@@ -243,7 +243,7 @@ class EventMethods(BaseClientAPI):
             https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidmembers
         """
         content = await self.api.request(
-            Method.GET, Path.rooms[room_id].joined_members, metrics_method="getJoinedMembers"
+            Method.GET, Path.v3.rooms[room_id].joined_members, metrics_method="getJoinedMembers"
         )
         try:
             return {
@@ -266,7 +266,7 @@ class EventMethods(BaseClientAPI):
         from_token: SyncToken,
         to_token: SyncToken | None = None,
         limit: int | None = None,
-        filter_json: str | None = None,
+        filter_json: str | dict | RoomEventFilter | None = None,
     ) -> PaginatedMessages:
         """
         Get a list of message and state events for a room. Pagination parameters are used to
@@ -291,16 +291,20 @@ class EventMethods(BaseClientAPI):
         .. _sync endpoint:
             https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3sync
         """
+        if isinstance(filter_json, Serializable):
+            filter_json = filter_json.json()
+        elif isinstance(filter_json, dict):
+            filter_json = json.dumps(filter_json)
         query_params = {
             "from": from_token,
             "dir": direction.value,
             "to": to_token,
             "limit": str(limit) if limit else None,
-            "filter_json": filter_json,
+            "filter": filter_json,
         }
         content = await self.api.request(
             Method.GET,
-            Path.rooms[room_id].messages,
+            Path.v3.rooms[room_id].messages,
             query_params=query_params,
             metrics_method="getMessages",
         )
@@ -352,7 +356,7 @@ class EventMethods(BaseClientAPI):
         content = content.serialize() if isinstance(content, Serializable) else content
         resp = await self.api.request(
             Method.PUT,
-            Path.rooms[room_id].state[event_type][state_key],
+            Path.v3.rooms[room_id].state[event_type][state_key],
             content,
             **kwargs,
             metrics_method="sendStateEvent",
@@ -392,7 +396,7 @@ class EventMethods(BaseClientAPI):
             raise ValueError("Room ID not given")
         elif not event_type:
             raise ValueError("Event type not given")
-        url = Path.rooms[room_id].send[event_type][txn_id or self.api.get_txn_id()]
+        url = Path.v3.rooms[room_id].send[event_type][txn_id or self.api.get_txn_id()]
         content = content.serialize() if isinstance(content, Serializable) else content
         resp = await self.api.request(
             Method.PUT, url, content, **kwargs, metrics_method="sendMessageEvent"
@@ -663,7 +667,7 @@ class EventMethods(BaseClientAPI):
         Returns:
             The ID of the event that was sent to redact the other event.
         """
-        url = Path.rooms[room_id].redact[event_id][self.api.get_txn_id()]
+        url = Path.v3.rooms[room_id].redact[event_id][self.api.get_txn_id()]
         content = extra_content or {}
         if reason:
             content["reason"] = reason
