@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -152,7 +152,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
                 else power_level_override
             )
 
-        resp = await self.api.request(Method.POST, Path.createRoom, content)
+        resp = await self.api.request(Method.POST, Path.v3.createRoom, content)
         try:
             return resp["room_id"]
         except KeyError:
@@ -179,12 +179,12 @@ class RoomMethods(EventMethods, BaseClientAPI):
         room_alias = f"#{alias_localpart}:{self.domain}"
         content = {"room_id": room_id}
         try:
-            await self.api.request(Method.PUT, Path.directory.room[room_alias], content)
+            await self.api.request(Method.PUT, Path.v3.directory.room[room_alias], content)
         except MatrixRequestError as e:
             if e.http_status == 409:
                 if override:
                     await self.remove_room_alias(alias_localpart)
-                    await self.api.request(Method.PUT, Path.directory.room[room_alias], content)
+                    await self.api.request(Method.PUT, Path.v3.directory.room[room_alias], content)
                 else:
                     raise MRoomInUse(e.http_status, e.message) from e
             else:
@@ -205,7 +205,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
         """
         room_alias = f"#{alias_localpart}:{self.domain}"
         try:
-            await self.api.request(Method.DELETE, Path.directory.room[room_alias])
+            await self.api.request(Method.DELETE, Path.v3.directory.room[room_alias])
         except MNotFound:
             if raise_404:
                 raise
@@ -226,7 +226,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
         Returns:
             The room ID and a list of servers that are aware of the room.
         """
-        content = await self.api.request(Method.GET, Path.directory.room[room_alias])
+        content = await self.api.request(Method.GET, Path.v3.directory.room[room_alias])
         return RoomAliasInfo.deserialize(content)
 
     # endregion
@@ -235,7 +235,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
 
     async def get_joined_rooms(self) -> list[RoomID]:
         """Get the list of rooms the user is in."""
-        content = await self.api.request(Method.GET, Path.joined_rooms)
+        content = await self.api.request(Method.GET, Path.v3.joined_rooms)
         try:
             return content["joined_rooms"]
         except KeyError:
@@ -273,7 +273,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
             return room_id
         content = await self.api.request(
             Method.POST,
-            Path.rooms[room_id].join,
+            Path.v3.rooms[room_id].join,
             {"third_party_signed": third_party_signed} if third_party_signed is not None else None,
         )
         try:
@@ -319,7 +319,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
             try:
                 content = await self.api.request(
                     Method.POST,
-                    Path.join[room_id_or_alias],
+                    Path.v3.join[room_id_or_alias],
                     content=content,
                     query_params=query_params,
                 )
@@ -395,7 +395,11 @@ class RoomMethods(EventMethods, BaseClientAPI):
         )
 
     async def invite_user(
-        self, room_id: RoomID, user_id: UserID, extra_content: dict[str, JSON] | None = None
+        self,
+        room_id: RoomID,
+        user_id: UserID,
+        reason: str | None = None,
+        extra_content: dict[str, JSON] | None = None,
     ) -> None:
         """
         Invite a user to participate in a particular room. They do not start participating in the
@@ -413,6 +417,8 @@ class RoomMethods(EventMethods, BaseClientAPI):
         Args:
             room_id: The ID of the room to which to invite the user.
             user_id: The fully qualified user ID of the invitee.
+            reason: The reason the user was invited. This will be supplied as the ``reason`` on
+                the `m.room.member`_ event.
             extra_content: Additional properties for the invite event content.
                 If a non-empty dict is passed, the invite event will be created using
                 the ``PUT /state/m.room.member/...`` endpoint instead of ``POST /invite``.
@@ -421,10 +427,11 @@ class RoomMethods(EventMethods, BaseClientAPI):
             await self.send_member_event(
                 room_id, user_id, Membership.INVITE, extra_content=extra_content
             )
-            return
-        await self.api.request(
-            Method.POST, Path.rooms[room_id].invite, content={"user_id": user_id}
-        )
+        else:
+            data = {"user_id": user_id}
+            if reason:
+                data["reason"] = reason
+            await self.api.request(Method.POST, Path.v3.rooms[room_id].invite, content=data)
 
     # endregion
     # region 8.4.2 Leaving rooms
@@ -433,6 +440,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
     async def leave_room(
         self,
         room_id: RoomID,
+        reason: str | None = None,
         extra_content: dict[str, JSON] | None = None,
         raise_not_in_room: bool = False,
     ) -> None:
@@ -453,6 +461,8 @@ class RoomMethods(EventMethods, BaseClientAPI):
 
         Args:
             room_id: The ID of the room to leave.
+            reason: The reason for leaving the room. This will be supplied as the ``reason`` on
+                the updated `m.room.member`_ event.
             extra_content: Additional properties for the leave event content.
                 If a non-empty dict is passed, the leave event will be created using
                 the ``PUT /state/m.room.member/...`` endpoint instead of ``POST /leave``.
@@ -464,7 +474,10 @@ class RoomMethods(EventMethods, BaseClientAPI):
                     room_id, self.mxid, Membership.LEAVE, extra_content=extra_content
                 )
             else:
-                await self.api.request(Method.POST, Path.rooms[room_id].leave)
+                data = {}
+                if reason:
+                    data["reason"] = reason
+                await self.api.request(Method.POST, Path.v3.rooms[room_id].leave, content=data)
         except MatrixRequestError as e:
             if "not in room" not in e.message or raise_not_in_room:
                 raise
@@ -485,7 +498,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
         Args:
             room_id: The ID of the room to forget.
         """
-        await self.api.request(Method.POST, Path.rooms[room_id].forget)
+        await self.api.request(Method.POST, Path.v3.rooms[room_id].forget)
 
     async def kick_user(
         self,
@@ -525,7 +538,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
             )
             return
         await self.api.request(
-            Method.POST, Path.rooms[room_id].kick, {"user_id": user_id, "reason": reason}
+            Method.POST, Path.v3.rooms[room_id].kick, {"user_id": user_id, "reason": reason}
         )
 
     # endregion
@@ -565,7 +578,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
             )
             return
         await self.api.request(
-            Method.POST, Path.rooms[room_id].ban, {"user_id": user_id, "reason": reason}
+            Method.POST, Path.v3.rooms[room_id].ban, {"user_id": user_id, "reason": reason}
         )
 
     async def unban_user(self, room_id: RoomID, user_id: UserID) -> None:
@@ -580,7 +593,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
             room_id: The ID of the room from which the user should be unbanned.
             user_id: The fully qualified user ID of the user being banned.
         """
-        await self.api.request(Method.POST, Path.rooms[room_id].unban, {"user_id": user_id})
+        await self.api.request(Method.POST, Path.v3.rooms[room_id].unban, {"user_id": user_id})
 
     # endregion
 
@@ -600,7 +613,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
         Returns:
             The visibility of the room in the directory.
         """
-        resp = await self.api.request(Method.GET, Path.directory.list.room[room_id])
+        resp = await self.api.request(Method.GET, Path.v3.directory.list.room[room_id])
         try:
             return RoomDirectoryVisibility(resp["visibility"])
         except KeyError:
@@ -627,7 +640,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
         """
         await self.api.request(
             Method.PUT,
-            Path.directory.list.room[room_id],
+            Path.v3.directory.list.room[room_id],
             {
                 "visibility": visibility.value,
             },
@@ -687,7 +700,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
         query_params = {"server": server} if server is not None else None
 
         content = await self.api.request(
-            method, Path.publicRooms, content, query_params=query_params
+            method, Path.v3.publicRooms, content, query_params=query_params
         )
 
         return RoomDirectoryResponse.deserialize(content)
