@@ -1,12 +1,11 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from __future__ import annotations
 
-from typing import Any
-import sys
+from typing import Any, TypedDict
 
 from mautrix.types import (
     Member,
@@ -20,11 +19,6 @@ from mautrix.types import (
 
 from .abstract import StateStore
 
-if sys.version_info >= (3, 8):
-    from typing import TypedDict
-else:
-    from typing_extensions import TypedDict
-
 
 class SerializedStateStore(TypedDict):
     members: dict[RoomID, dict[UserID, Any]]
