@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -175,15 +175,12 @@ async def test_updates(request, store: StateStore) -> None:
     assert set(await store.get_members(room_id)) == joined_members
     assert set(await store.get_members(room_id, memberships=any_membership)) == full_members
     assert set(await store.get_members(room_id, memberships=leave_memberships)) == left_members
-    assert (
-        set(
-            await store.get_members_filtered(
-                room_id,
-                memberships=leave_memberships,
-                not_id="",
-                not_prefix="@telegram_",
-                not_suffix=":example.com",
-            )
+    assert set(
+        await store.get_members_filtered(
+            room_id,
+            memberships=leave_memberships,
+            not_id="",
+            not_prefix="@telegram_",
+            not_suffix=":example.com",
         )
-        == {"@whatsappbot:example.com"}
-    )
+    ) == {"@whatsappbot:example.com"}
