@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,7 +7,7 @@ from __future__ import annotations
 
 import asyncio
 
-from mautrix.errors import MNotFound
+from mautrix.errors import MForbidden, MNotFound
 from mautrix.types import (
     JSON,
     EventID,
@@ -69,10 +69,11 @@ class StoreUpdatingAPI(ClientAPI):
     async def leave_room(
         self,
         room_id: RoomID,
+        reason: str | None = None,
         extra_content: dict[str, JSON] | None = None,
         raise_not_in_room: bool = False,
     ) -> None:
-        await super().leave_room(room_id, extra_content, raise_not_in_room)
+        await super().leave_room(room_id, reason, extra_content, raise_not_in_room)
         if not extra_content and self.state_store:
             await self.state_store.set_membership(room_id, self.mxid, Membership.LEAVE)
 
@@ -80,9 +81,10 @@ class StoreUpdatingAPI(ClientAPI):
         self,
         room_id: RoomID,
         user_id: UserID,
+        reason: str | None = None,
         extra_content: dict[str, JSON] | None = None,
     ) -> None:
-        await super().invite_user(room_id, user_id, extra_content=extra_content)
+        await super().invite_user(room_id, user_id, reason, extra_content=extra_content)
         if not extra_content and self.state_store:
             await self.state_store.set_membership(room_id, user_id, Membership.INVITE)
 
@@ -238,7 +240,7 @@ class StoreUpdatingAPI(ClientAPI):
 
             try:
                 profile = await self.get_profile(user_id)
-            except MNotFound:
+            except (MNotFound, MForbidden):
                 profile = None
             if profile:
                 self.log.trace(
