@@ -1,15 +1,14 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from __future__ import annotations
 
-from typing import Any, Awaitable, Callable, Dict
+from typing import Any, Awaitable, Callable, Dict, TypedDict
 import asyncio
 import functools
 import json
-import sys
 
 import olm
 
@@ -27,11 +26,6 @@ from mautrix.util.logging import TraceLogger
 
 from .. import client as cli, crypto
 
-if sys.version_info >= (3, 8):
-    from typing import TypedDict
-else:
-    from typing_extensions import TypedDict
-
 
 class SignedObject(TypedDict):
     signatures: Dict[UserID, Dict[str, str]]
