@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,10 +6,17 @@
 from typing import Dict, List, Optional
 
 from mautrix.errors import DeviceValidationError
-from mautrix.types import DeviceID, DeviceKeys, IdentityKey, SyncToken, UserID
+from mautrix.types import (
+    DeviceID,
+    DeviceIdentity,
+    DeviceKeys,
+    IdentityKey,
+    SyncToken,
+    TrustState,
+    UserID,
+)
 
 from .base import BaseOlmMachine, verify_signature_json
-from .types import DeviceIdentity, TrustState
 
 
 class DeviceListMachine(BaseOlmMachine):
