@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,6 +13,7 @@ import time
 from mautrix.errors import EncryptionError, SessionShareError
 from mautrix.types import (
     DeviceID,
+    DeviceIdentity,
     EncryptedMegolmEventContent,
     EncryptionAlgorithm,
     EventType,
@@ -24,13 +25,13 @@ from mautrix.types import (
     Serializable,
     SessionID,
     SigningKey,
+    TrustState,
     UserID,
 )
 
 from .device_lists import DeviceListMachine
 from .encrypt_olm import OlmEncryptionMachine
 from .sessions import InboundGroupSession, OutboundGroupSession, Session
-from .types import DeviceIdentity, TrustState
 
 
 class Sentinel:
