@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@ from typing import Optional
 
 from mautrix.errors import MatrixConnectionError, MatrixError, MatrixRequestError
 from mautrix.types import (
+    DeviceIdentity,
     EncryptionAlgorithm,
     EventType,
     ForwardedRoomKeyEventContent,
@@ -16,11 +17,11 @@ from mautrix.types import (
     RoomKeyWithheldCode,
     RoomKeyWithheldEventContent,
     ToDeviceEvent,
+    TrustState,
 )
 
 from .device_lists import DeviceListMachine
 from .encrypt_olm import OlmEncryptionMachine
-from .types import DeviceIdentity, TrustState
 
 
 class RejectKeyShare(MatrixError):
@@ -64,9 +65,7 @@ class KeySharingMachine(OlmEncryptionMachine, DeviceListMachine):
         """
         if device.user_id != self.client.mxid:
             raise RejectKeyShare(
-                f"Rejecting key request from a different user ({device.user_id})",
-                code=RoomKeyWithheldCode.UNAUTHORIZED,
-                reason="This device does not share keys to other users",
+                f"Ignoring key request from a different user ({device.user_id})", code=None
             )
         elif device.device_id == self.client.device_id:
             raise RejectKeyShare("Ignoring key request from ourselves", code=None)
