@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,6 +11,7 @@ import logging
 
 from mautrix import client as cli
 from mautrix.types import (
+    DecryptedOlmEvent,
     DeviceLists,
     DeviceOTKCount,
     EncryptionAlgorithm,
@@ -27,7 +28,6 @@ from .encrypt_megolm import MegolmEncryptionMachine
 from .key_request import KeyRequestingMachine
 from .key_share import KeySharingMachine
 from .store import CryptoStore, StateStore
-from .types import DecryptedOlmEvent
 from .unwedge import OlmUnwedgingMachine
 
 
