@@ -1,28 +1,30 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from __future__ import annotations
 
-from typing import cast
 from collections import defaultdict
 from datetime import timedelta
 
 from mautrix.client.state_store import SyncStore
 from mautrix.client.state_store.asyncpg import PgStateStore
-from mautrix.types import DeviceID, EventID, IdentityKey, RoomID, SessionID, SyncToken, UserID
-from mautrix.util.async_db import Database
+from mautrix.types import (
+    DeviceID,
+    DeviceIdentity,
+    EventID,
+    IdentityKey,
+    RoomID,
+    SessionID,
+    SyncToken,
+    TrustState,
+    UserID,
+)
+from mautrix.util.async_db import Database, Scheme
 from mautrix.util.logging import TraceLogger
 
-from ... import (
-    DeviceIdentity,
-    InboundGroupSession,
-    OlmAccount,
-    OutboundGroupSession,
-    Session,
-    TrustState,
-)
+from ... import InboundGroupSession, OlmAccount, OutboundGroupSession, Session
 from ..abstract import CryptoStore, StateStore
 from .upgrade import upgrade_table
 
@@ -273,7 +275,7 @@ class PgCryptoStore(CryptoStore, SyncStore):
     async def add_outbound_group_session(self, session: OutboundGroupSession) -> None:
         pickle = session.pickle(self.pickle_key)
         max_age = session.max_age
-        if self.db.scheme == "sqlite":
+        if self.db.scheme == Scheme.SQLITE:
             max_age = max_age.total_seconds()
         await self.db.execute(
             "INSERT INTO crypto_megolm_outbound_session (room_id, session_id, session, shared, "
@@ -317,7 +319,7 @@ class PgCryptoStore(CryptoStore, SyncStore):
         if row is None:
             return None
         max_age = row["max_age"]
-        if self.db.scheme == "sqlite":
+        if self.db.scheme == Scheme.SQLITE:
             max_age = timedelta(seconds=max_age)
         return OutboundGroupSession.from_pickle(
             row["session"],
@@ -339,7 +341,7 @@ class PgCryptoStore(CryptoStore, SyncStore):
         )
 
     async def remove_outbound_group_sessions(self, rooms: list[RoomID]) -> None:
-        if self.db.scheme == "postgres":
+        if self.db.scheme in (Scheme.POSTGRES, Scheme.COCKROACH):
             await self.db.execute(
                 "DELETE FROM crypto_megolm_outbound_session "
                 "WHERE account_id=$1 AND room_id=ANY($2)",
@@ -374,7 +376,7 @@ class PgCryptoStore(CryptoStore, SyncStore):
         index: int,
         timestamp: int,
     ) -> bool:
-        if self.db.scheme == "postgres":
+        if self.db.scheme in (Scheme.POSTGRES, Scheme.COCKROACH):
             row = await self.db.fetchrow(
                 self._validate_message_index_query,
                 sender_key,
@@ -499,7 +501,7 @@ class PgCryptoStore(CryptoStore, SyncStore):
                 user_id,
             )
             await conn.execute("DELETE FROM crypto_device WHERE user_id=$1", user_id)
-            if self.db.scheme == "postgres":
+            if self.db.scheme == Scheme.POSTGRES:
                 await conn.copy_records_to_table("crypto_device", records=data, columns=columns)
             else:
                 await conn.executemany(
@@ -510,7 +512,7 @@ class PgCryptoStore(CryptoStore, SyncStore):
                 )
 
     async def filter_tracked_users(self, users: list[UserID]) -> list[UserID]:
-        if self.db.scheme == "postgres":
+        if self.db.scheme in (Scheme.POSTGRES, Scheme.COCKROACH):
             rows = await self.db.fetch(
                 "SELECT user_id FROM crypto_tracked_user WHERE user_id = ANY($1)", users
             )
