@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -31,7 +31,7 @@ class RoomCreatePreset(Enum):
     Room creation preset, as specified in the `createRoom endpoint`_
 
     .. _createRoom endpoint:
-        https://spec.matrix.org/v1.1/client-server-api/#post_matrixclientv3createroom
+        https://spec.matrix.org/v1.2/client-server-api/#post_matrixclientv3createroom
     """
 
     PRIVATE = "private_chat"
@@ -44,7 +44,7 @@ class RoomDirectoryVisibility(Enum):
     Room directory visibility, as specified in the `createRoom endpoint`_
 
     .. _createRoom endpoint:
-        https://spec.matrix.org/v1.1/client-server-api/#post_matrixclientv3createroom
+        https://spec.matrix.org/v1.2/client-server-api/#post_matrixclientv3createroom
     """
 
     PRIVATE = "private"
@@ -64,7 +64,7 @@ class RoomAliasInfo(SerializableAttrs):
     Room alias query result, as specified in the `alias resolve endpoint`_
 
     .. _alias resolve endpoint:
-        https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3directoryroomroomalias
+        https://spec.matrix.org/v1.2/client-server-api/#get_matrixclientv3directoryroomroomalias
     """
 
     room_id: RoomID = None
@@ -112,6 +112,14 @@ class VersionsResponse(SerializableAttrs):
     versions: List[str]
     unstable_features: Dict[str, bool] = attr.ib(factory=lambda: {})
 
+    @property
+    def has_legacy_versions(self) -> bool:
+        return any(v for v in self.versions if v.startswith("r0."))
+
+    @property
+    def has_modern_versions(self) -> bool:
+        return any(v for v in self.versions if v.startswith("v"))
+
 
 @dataclass
 class BatchSendResponse(SerializableAttrs):
