@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,15 +7,17 @@ from __future__ import annotations
 
 from typing import Any
 from contextlib import asynccontextmanager
-from urllib.parse import urlparse
 import asyncio
 import logging
 import re
 import sqlite3
 
+from yarl import URL
 import aiosqlite
 
+from .connection import LoggingConnection
 from .database import Database
+from .scheme import Scheme
 from .upgrade import UpgradeTable
 
 POSITIONAL_PARAM_PATTERN = re.compile(r"\$(\d+)")
@@ -45,12 +47,16 @@ class TxnConnection(aiosqlite.Connection):
         query = POSITIONAL_PARAM_PATTERN.sub(r"?\1", query)
         return super().execute(query, args)
 
-    async def execute(self, query: str, *args: Any, timeout: float | None = None) -> None:
-        await self.__execute(query, *args)
+    async def execute(
+        self, query: str, *args: Any, timeout: float | None = None
+    ) -> aiosqlite.Cursor:
+        return await self.__execute(query, *args)
 
-    async def executemany(self, query: str, *args: Any, timeout: float | None = None) -> None:
+    async def executemany(
+        self, query: str, *args: Any, timeout: float | None = None
+    ) -> aiosqlite.Cursor:
         query = POSITIONAL_PARAM_PATTERN.sub(r"?\1", query)
-        await super().executemany(query, *args)
+        return await super().executemany(query, *args)
 
     async def fetch(
         self, query: str, *args: Any, timeout: float | None = None
@@ -58,7 +64,9 @@ class TxnConnection(aiosqlite.Connection):
         async with self.__execute(query, *args) as cursor:
             return list(await cursor.fetchall())
 
-    async def fetchrow(self, query: str, *args: Any, timeout: float | None = None) -> sqlite3.Row:
+    async def fetchrow(
+        self, query: str, *args: Any, timeout: float | None = None
+    ) -> sqlite3.Row | None:
         async with self.__execute(query, *args) as cursor:
             return await cursor.fetchone()
 
@@ -72,23 +80,32 @@ class TxnConnection(aiosqlite.Connection):
 
 
 class SQLiteDatabase(Database):
-    scheme = "sqlite"
+    scheme = Scheme.SQLITE
     _pool: asyncio.Queue[TxnConnection]
     _stopped: bool
     _conns: int
 
     def __init__(
         self,
-        url: str,
+        url: URL,
         upgrade_table: UpgradeTable,
         db_args: dict[str, Any] | None = None,
         log: logging.Logger | None = None,
+        owner_name: str | None = None,
+        ignore_foreign_tables: bool = True,
     ) -> None:
-        super().__init__(url, db_args=db_args, upgrade_table=upgrade_table, log=log)
-        self._path = urlparse(url).path
+        super().__init__(
+            url,
+            db_args=db_args,
+            upgrade_table=upgrade_table,
+            log=log,
+            owner_name=owner_name,
+            ignore_foreign_tables=ignore_foreign_tables,
+        )
+        self._path = url.path
         if self._path.startswith("/"):
             self._path = self._path[1:]
-        self._pool = asyncio.Queue(self._db_args.pop("min_size", 5))
+        self._pool = asyncio.Queue(self._db_args.pop("min_size", 1))
         self._db_args.pop("max_size", None)
         self._stopped = False
         self._conns = 0
@@ -110,12 +127,12 @@ class SQLiteDatabase(Database):
             await conn.close()
 
     @asynccontextmanager
-    async def acquire(self) -> TxnConnection:
+    async def acquire(self) -> LoggingConnection:
         if self._stopped:
             raise RuntimeError("database pool has been stopped")
         conn = await self._pool.get()
         try:
-            yield conn
+            yield LoggingConnection(self.scheme, conn, self.log)
         finally:
             self._pool.put_nowait(conn)
 
