@@ -1,9 +1,11 @@
-# Copyright (c) 2021 Tulir Asokan
+# Copyright (c) 2022 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Any, AsyncIterable, Awaitable, Iterable, Optional, Tuple, Type, Union
+from __future__ import annotations
+
+from typing import Any, AsyncIterable, Awaitable, Iterable, Union, cast
 from itertools import chain
 from time import time
 import argparse
@@ -16,6 +18,7 @@ import signal
 import sys
 
 from .config import BaseFileConfig, BaseMissingError, BaseValidatableConfig, ConfigValueError
+from .logging import TraceLogger
 
 try:
     import uvloop
@@ -39,11 +42,11 @@ class Program:
     """
 
     loop: asyncio.AbstractEventLoop
-    log: logging.Logger
+    log: TraceLogger
     parser: argparse.ArgumentParser
     args: argparse.Namespace
 
-    config_class: Type[BaseFileConfig]
+    config_class: type[BaseFileConfig]
     config: BaseFileConfig
 
     startup_actions: TaskList
@@ -57,12 +60,12 @@ class Program:
 
     def __init__(
         self,
-        module: Optional[str] = None,
-        name: Optional[str] = None,
-        description: Optional[str] = None,
-        command: Optional[str] = None,
-        version: Optional[str] = None,
-        config_class: Optional[Type[BaseFileConfig]] = None,
+        module: str | None = None,
+        name: str | None = None,
+        description: str | None = None,
+        command: str | None = None,
+        version: str | None = None,
+        config_class: type[BaseFileConfig] | None = None,
     ) -> None:
         if module:
             self.module = module
@@ -174,7 +177,7 @@ class Program:
     def prepare_log(self) -> None:
         """Pre-init lifecycle method. Extend this if you want to customize logging setup."""
         logging.config.dictConfig(copy.deepcopy(self.config["logging"]))
-        self.log = logging.getLogger("mau.init")
+        self.log = cast(TraceLogger, logging.getLogger("mau.init"))
 
     def prepare(self) -> None:
         """
@@ -186,7 +189,7 @@ class Program:
     def prepare_loop(self) -> None:
         """Init lifecycle method where the asyncio event loop is created."""
         if uvloop is not None:
-            asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+            uvloop.install()
             self.log.debug("Using uvloop for asyncio")
 
         self.loop = asyncio.get_event_loop()
@@ -226,13 +229,20 @@ class Program:
             self.log.debug("Interrupt received, stopping...")
         except Exception:
             self.log.critical("Unexpected error in main event loop", exc_info=True)
+            self.loop.run_until_complete(self.system_exit())
             sys.exit(2)
+        except SystemExit:
+            self.loop.run_until_complete(self.system_exit())
+            raise
         self.prepare_stop()
         self.loop.run_until_complete(self.stop())
         self.prepare_shutdown()
         self.log.info("Everything stopped, shutting down")
         sys.exit(0)
 
+    async def system_exit(self) -> None:
+        """Lifecycle method that is called if the main event loop exits using ``sys.exit()``."""
+
     async def start(self) -> None:
         """
         First lifecycle method called inside the asyncio event loop. Extend this if you want more
@@ -276,7 +286,7 @@ class Program:
                 tasks.append(asyncio.create_task(task))
         await asyncio.gather(*tasks)
 
-    def _add_actions(self, to: TaskList, add: Tuple[NewTask, ...]) -> TaskList:
+    def _add_actions(self, to: TaskList, add: tuple[NewTask, ...]) -> TaskList:
         for item in add:
             if inspect.isasyncgen(item):
                 to.append(self._unpack_async_iterator(item))
